/*
 * This file is part of din.
 *
 * din is copyright (c) 2006 - 2011 S Jagannathan <jag@dinisnoise.org>
 * For more information, please visit http://dinisnoise.org
 *
 * din is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * din is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with din.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
#ifndef __AUDIO
#define __AUDIO

#include <jack/jack.h>
#include <jack/midiport.h>
#include <jack/ringbuffer.h>
#include "console.h"
#include <iostream>

typedef jack_default_audio_sample_t sample_t; // this is just float in jack/types.h

struct audio_out {

	static const int num_channels = 2;
	int samples_per_channel;
	int samples_per_buffer; // has 2 channels
	int samples_buffer_size;
	sample_t sample_rate;

	// written by the main thread
	//
	// a multi buffer scheme
	//
	// main thread writes a buffer, while audio thread reads another
	//

	float* samples_buffers; // a bunch of sample buffers
	int num_samples_buffers;
	float *readp, *writep;
	int readi, writei;
	int* available; // buffer available for audio out?

	// reuseable buffers
	float *result, *ams, *fms; // result, AM & FM
	float *step, *left, *right, *vol;

	//
	// jack
	//

	jack_client_t *client;

	// L and R connections
	int lcons, rcons; // l and r connected?
	jack_port_t *out_left, *out_right;
	const char **con_left, **con_right;

	jack_port_t *midi_in;
	jack_ringbuffer_t *midi_rb;

	std::string prefs_name;
	void load_prefs ();
	void save_prefs ();
	void defaults ();

	audio_out ();
	~audio_out ();

	int open ();
	int close ();
	int start ();
	int stop ();
	void write (float* abuf, float* lbuf, float* rbuf);
	void alloc ();
	void set_sample_rate (int s);

};

int audio_wanted (jack_nframes_t nsamples, void *arg);
int buffer_size_changed (jack_nframes_t nframes, void *arg);
int sample_rate_changed (jack_nframes_t nframes, void *arg);
void process_jackmidi ();

console& operator<< (console& c, const audio_out& ao);

#endif
