/*
 * This file is part of din.
 *
 * din is copyright (c) 2006 - 2011 S Jagannathan <jag@dinisnoise.org>
 * For more information, please visit http://dinisnoise.org
 *
 * din is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * din is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with din.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
#ifndef __input
#define __input

#include "chrono.h"
#include <SDL/SDL.h>

struct keyboard {

		int nkeys; // number of keys
		Uint8* now; // key state now (1 - down, 0 - up)
    Uint8* last; // last state of keys
    unsigned int* querky; // list of keys whose state was queried
    int nquer; // num of queried keys

    double* tstamp;
    double* repeat;

		keyboard () {

			SDL_GetKeyState (&nkeys);

      last = new Uint8 [nkeys];

      querky = new unsigned int [nkeys];
      nquer = 0;

      tstamp = new double [nkeys];
      repeat = new double [nkeys];

		}

		void read () { // current keyboard state
			now = SDL_GetKeyState (0);
		}

		void save () {
		  for (int i = 0; i < nquer; ++i) {
		    unsigned int k = querky[i];
		    last[k] = now[k];
      }
      nquer = 0;
    }

		~keyboard () {
      delete[] last;
		  delete[] querky;
      delete[] tstamp;
      delete[] repeat;
    }

};

extern keyboard keybd;

inline int nkeys () {
	return keybd.nkeys;
}

inline int keydown (unsigned int k) {
  keybd.querky [keybd.nquer++] = k;
  return keybd.now[k];
}

inline int keyup (unsigned int k) {
  int wkd = keybd.last[k];
  int kn = keybd.now[k];
  return (!kn && wkd);
}

inline int waskeydown (unsigned int k) {
	return keybd.last[k];
}

inline int keypressed (unsigned int k) {
	int wkd = waskeydown (k), kd = keydown (k);
	if (kd && !wkd) return 1; else return 0;
}

extern nano_chrono ui_clk;

inline int keypressedd (unsigned int k, double firstt = 1./3, double constt = 1./20) {

	/* keypressed with key repeat */

	int wkd = waskeydown(k), kd = keydown(k);
	if (kd) {
		if (!wkd) {
			keybd.repeat[k] = firstt;
			keybd.tstamp[k] = ui_clk.secs;
			return 1;
		} else {
			double dt = ui_clk.secs - keybd.tstamp[k];
			if (dt > keybd.repeat[k]) {
				keybd.repeat[k] = constt;
				keybd.tstamp[k] = ui_clk.secs;
				return 1;
			}
		}
	}
	return 0;
}

#endif
