/*
 * This file is part of din.
 *
 * din is copyright (c) 2006 - 2011 S Jagannathan <jag@dinisnoise.org>
 * For more information, please visit http://dinisnoise.org
 *
 * din is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * din is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with din.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
#include "din.h"
#include "tokenizer.h"

string APP_NAME;
int SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_DEPTH;

int SAMPLE_RATE;
double SECS_PER_BUFFER;
float MIN_TONIC = 40, MAX_TONIC = 11025;

string TUNING;
string INTERVALS_FILE;
int NUM_INTERVALS;
map <string, float> INTERVALS;
map <string, int> NOTE_POS;
string FIRST_INTERVAL_NAME, LAST_INTERVAL_NAME;

int NUM_VOLUMES, LAST_VOLUME;
float DELTA_VOLUME;

int NUM_MICROTONES;
int LEFT, BOTTOM, RIGHT, TOP, HEIGHT;

string SCALE;

int FPS, USLEEP;
double FPS_T;

void calc_tonic_limits () {
	MIN_TONIC = 40; // hz
	MAX_TONIC = SAMPLE_RATE / 4;
}

void calc_volume_vars (int h) {
	NUM_VOLUMES = h;
	DELTA_VOLUME = 1./NUM_VOLUMES;
	LAST_VOLUME = NUM_VOLUMES - 1;
}

bool load_globals::load_intervals (const string& fname) {

	extern string dotdin;
	ifstream file ((dotdin + fname).c_str(), ios::in);
	if (file) {

		INTERVALS_FILE = fname;

		file >> ignore >> NUM_INTERVALS;
		for (int i = 0, last = NUM_INTERVALS - 1, pos = 0; i < NUM_INTERVALS; ++i, ++pos) {

			string note_name;
			float value;

			file >> note_name >> value;

			if (i == 0) FIRST_INTERVAL_NAME = note_name;
			else if (i == last) LAST_INTERVAL_NAME = note_name;

			INTERVALS [note_name] = value;
			NOTE_POS [note_name] = pos;

		}

		return true;


	} else return false;

}

load_globals::load_globals () {
	extern string dotdin;
	string fname = dotdin + "globals";
	ifstream file (fname.c_str(), ios::in);
	if (file) {
		cout << "<<< loading globals from: " << fname;
		file >> ignore >> APP_NAME;
		file >> ignore >> SCREEN_WIDTH >> SCREEN_HEIGHT >> SCREEN_DEPTH;
		file >> ignore >> INTERVALS_FILE;
		file >> ignore >> SCALE;
		file >> ignore >> LEFT >> BOTTOM;
		file >> ignore >> HEIGHT;
		file >> ignore >> NUM_MICROTONES;
		file >> ignore >> viewport::HANDLE_RADIUS;

		cout << ", done >>>" << endl;
	} else {
		cout << "bad globals file, will use defaults " << endl;
		APP_NAME = "din";
		SCREEN_WIDTH = 640; SCREEN_HEIGHT = 480;
		INTERVALS_FILE = "pytha.tuning";
		SCALE = "major";
		LEFT = 0; BOTTOM = 150; HEIGHT = 300;
		NUM_MICROTONES = 150;
	}

	load_intervals (INTERVALS_FILE);
	TOP = BOTTOM + HEIGHT;
	calc_volume_vars (HEIGHT);

}




